/*
	SuperCollider real time audio synthesis system
 Copyright (c) 2002 James McCartney. All rights reserved.
	http://www.audiosynth.com
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//ACMC demo UGen

#include "SC_PlugIn.h"

static InterfaceTable *ft; 

struct ACMCdemo : public Unit  {
	float initvariable, storedvariable;
	float* buffer; 	
};

extern "C" {  
	
	void ACMCdemo_next(ACMCdemo *unit, int inNumSamples);
	void ACMCdemo_Ctor(ACMCdemo* unit);
	void ACMCdemo_Dtor(ACMCdemo* unit);
}


void ACMCdemo_Ctor( ACMCdemo* unit ) {
	
	unit->initvariable= ZIN0(1);	
	unit->storedvariable = 0.3;
	
	unit->buffer= (float*)RTAlloc(unit->mWorld, 64* sizeof(float));
	
	RGen& rgen = *unit->mParent->mRGen;
	
	int i=0;
	for(i=0; i<64;++i) {
		unit->buffer[i]=2*rgen.frand() - 1.0;
	}
	
	SETCALC(ACMCdemo_next);
}

void ACMCdemo_Dtor(ACMCdemo *unit) {
	RTFree(unit->mWorld, unit->buffer);
}

void ACMCdemo_next( ACMCdemo *unit, int inNumSamples ) {
	
	int i;
	float z;
		 
	float *in = IN(0);
	float *out = ZOUT(0);
	
	float stored= unit->storedvariable;
	//float init= unit->initvariable;
	
	float init= ZIN0(1);
	
	float * buffer = unit->buffer;
	
	for (i=0; i<inNumSamples; ++i) { 
		 
		 stored= fmod(stored*init*init+init,1.0);
		 
		//printf("%f \t", stored);
		 
		 z= in[i] * stored;
		 
		 if (z>buffer[i]) z*= (-0.5);
		 
		 ZXP(out) = z*0.0;
		 }
	
	unit->storedvariable = stored;
}	






extern "C" void load(InterfaceTable *inTable) {
	
	ft = inTable;
	
	DefineDtorUnit(ACMCdemo);
	
}


